/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components;

import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;

public abstract class AbstractComponentMonitoringSubFunction extends AbstractMonitoringSubFunction
        implements MonitoringSubFunction {

    /**
     * The option to get the component name
     */
    public static final String COMPONENT_NAME_LONG_OPTION = "component-name";

    public static final String COMPONENT_NAME_SHORT_OPTION = "n";

    protected static final Option COMPONENT_NAME_OPTION = Option.builder(COMPONENT_NAME_SHORT_OPTION).required(true)
            .numberOfArgs(1).argName("component-name").desc("The identifier of the component.")
            .longOpt(COMPONENT_NAME_LONG_OPTION).build();

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    protected Shell shell;

    protected AbstractComponentMonitoringSubFunction() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    protected void createOptions() {
        this.options = new Options();
        this.options.addOption(COMPONENT_NAME_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }
}
