/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringObject;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;

/**
 * Abstract base class of all monitored component. Used to declare probes
 * provided by CDK
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractMonitoredComponent implements MonitoringObject {

    private final static MonitoringSubFunction[] CDK_SUBFUNCTIONS = new MonitoringSubFunction[] {
            new ServiceProviderInvocationsCount(), new ServiceProviderInvocationsResponseTimes(),
            new MexAcceptorPool(), new MexAcceptors(), new MexProcessorThreadPool() };

    @Override
    public List<MonitoringSubFunction> getSubFunctions() {
        final List<MonitoringSubFunction> subFunctions = new ArrayList<>();
        subFunctions.addAll(Arrays.asList(CDK_SUBFUNCTIONS));
        final List<MonitoringSubFunction> dedicatedSubFunctions = this.getDedicatedSubFunctions();
        if (dedicatedSubFunctions != null) {
            subFunctions.addAll(dedicatedSubFunctions);
        }
        return subFunctions;
    }

    /**
     * @return the list of dedicated sub-functions of the component provided by
     *         the monitoring object. Can be <code>null</code> is no dedicated
     *         sub-function is provided.
     */
    public abstract List<MonitoringSubFunction> getDedicatedSubFunctions();

}
