/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringObject;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;

/**
 * Some assertions to use in unit tests of the Petals CLI monitoring extension
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class Assert {

    /**
     * Assert that the given monitoring object is provided with the right expected sub-functions
     * 
     * @param monitoringObj
     *            The monitoring object to check
     * @param originalExpectedSubFunctions
     *            The expected sub-functions
     */
    public final static void assertMonitoringSubFunctions(final MonitoringObject monitoringObj,
            final List<Class<? extends MonitoringSubFunction>> originalExpectedSubFunctions) {

        // The expected sub-function list is duplicated because we remove item from the list during checking, and if the
        // given list is static some problem can occur.
        final List<Class<? extends MonitoringSubFunction>> expectedSubFunctions = new ArrayList<>(
                originalExpectedSubFunctions);

        final List<MonitoringSubFunction> monitoringSubFunctions = new ArrayList<>(monitoringObj.getSubFunctions());
        final Iterator<MonitoringSubFunction> itMonitoringSubFunctions = monitoringSubFunctions.iterator();
        while (itMonitoringSubFunctions.hasNext()) {
            final MonitoringSubFunction monitoringObject = itMonitoringSubFunctions.next();

            final Iterator<Class<? extends MonitoringSubFunction>> itExpectedMonitoringSubFunctions = expectedSubFunctions
                    .iterator();
            while (itExpectedMonitoringSubFunctions.hasNext()) {
                final Class<? extends MonitoringSubFunction> expectedMonitoringSubFunction = itExpectedMonitoringSubFunctions
                        .next();
                if (expectedMonitoringSubFunction.isInstance(monitoringObject)) {
                    itExpectedMonitoringSubFunctions.remove();
                    itMonitoringSubFunctions.remove();
                    break;
                }
            }
        }

        assertEquals("Few monitoring sub-functions are not expected", 0, monitoringSubFunctions.size());
        assertEquals("Few monitoring sub-functions are missing", 0, expectedSubFunctions.size());
    }

}
