/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_ACTIVE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_ACTIVE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_ENQUEUED_REQUEST_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_ENQUEUED_REQUEST_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_IDLE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_IDLE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_MAX_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.HTTP_SERVER_THREAD_POOL_MIN_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INFORMATION_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.OUTGOING_WS_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.OUTGOING_WS_REQUEST_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.SERVICE_CONTRACTS_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.UNKNOWN_REQUEST_COUNT;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Before;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractMonitoringSubFunctionTest;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.OutgoingWsRequestKey;

/**
 * Abstract testcase for BC SOAP unit tests of command 'monitoring' for Cacti
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractTestCase extends AbstractMonitoringSubFunctionTest {

    /**
     * Clear all datasets used during unit tests
     * 
     * @throws Exception
     */
    @Before
    public void clearDataSets() throws Exception {
        INFORMATION_REQUEST_COUNT = null;
        UNKNOWN_REQUEST_COUNT = null;
        SERVICE_CONTRACTS_REQUEST_COUNT = null;
        INCOMING_WS_REQUEST_COUNT = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        OUTGOING_WS_REQUEST_COUNT = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        HTTP_SERVER_THREAD_POOL_ACTIVE_THREAD_MAX = null;
        HTTP_SERVER_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;
        HTTP_SERVER_THREAD_POOL_IDLE_THREAD_MAX = null;
        HTTP_SERVER_THREAD_POOL_IDLE_THREAD_CURRENT = null;
        HTTP_SERVER_THREAD_POOL_MAX_SIZE = null;
        HTTP_SERVER_THREAD_POOL_MIN_SIZE = null;
        HTTP_SERVER_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = null;
        HTTP_SERVER_THREAD_POOL_ENQUEUED_REQUEST_MAX = null;
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiIncomingResponseTimesOutput(final String output,
            final Map<IncomingWsRequestKey, Long[]> dataSet, final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {


        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long[]> entry : dataSet
                    .entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiOutgoingResponseTimesOutput(final String output,
            final Map<OutgoingWsRequestKey, Long[]> dataSet, final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<OutgoingWsRequestKey, Long[]> entry : dataSet.entrySet()) {
                final OutgoingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@"
                        + key.getExternalWsUrl())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    protected void assertCactiResponseTimesOutputOfQueryWithArg(final String output,
            final Map<String, Long> expectedResults, final String queriedOperation)
            throws NumberFormatException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiIncomingCountOutput(final String output,
            final Map<IncomingWsRequestKey, Long> dataSet, final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long> entry : dataSet.entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiOutgoingCountOutput(final String output,
            final Map<OutgoingWsRequestKey, Long> dataSet, final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<OutgoingWsRequestKey, Long> entry : dataSet
                    .entrySet()) {
                final OutgoingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@"
                        + key.getExternalWsUrl())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }
}
