/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOCATIONS_COUNT;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Before;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractMonitoringSubFunctionTest;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;

/**
 * Abstract testcase for component unit tests of command 'monitoring' for Cacti
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractTestCase extends AbstractMonitoringSubFunctionTest {

    /**
     * Clear all datasets used during unit tests
     * 
     * @throws Exception
     */
    @Before
    public void clearDataSets() throws Exception {
        SERVICE_PROVIDER_INVOCATIONS_COUNT = null;
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiServiceProviderInvocationResponseTimesOutput(final String output,
            final Map<ServiceProviderInvocationKey, Long[]> dataSet,
            final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {


        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long[]> entry : dataSet
                    .entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (operationNameReturned.equals(key.getOperation() + "@" + key.getServiceName()
                        + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    protected void assertCactiResponseTimesOutputOfQueryWithArg(final String output,
            final Map<String, Long> expectedResults, final String queriedOperation)
            throws NumberFormatException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    protected void assertCactiServiceProviderInvocationsCountOutput(final String output,
            final Map<ServiceProviderInvocationKey, Long> dataSet,
            final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : dataSet.entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (operationNameReturned.equals(key.getOperation() + "@" + key.getServiceName()
                        + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected succeeded value",
                            expectedResults.get(operationNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }
}
